<?php
/*
Plugin Name: Peter's Random Anti-Spam
Plugin URI: http://www.theblog.ca/random-anti-spam
Description: Stop a lot of spambots from polluting your site by making visitors identify some random text displayed as an image before commenting. From the creator of <a href="http://www.theblog.ca/anti-spam">Peter's Custom Anti-Spam</a>.
Author: Peter Keung
Version: 1.0.3
Author URI: http://www.theblog.ca
Change Log:
2008-05-24  Version 1.0.3  For efficiency reasons, image and audio generation is now run directly through this file.
2008-03-13  Version 1.0.2  Minor code tweak to be compatible with WordPress 2.5.
2008-02-10  Version 1.0.1  New option to disable random stretching of the wav files. Made the operation to stretch the wav files silent to the server(thanks Angelo!). Blocked indexing of audio file links by search engines (thanks Lucky!).
2008-01-20  Version 1.0.0  Added registration form protection (WordPress 2.1 and higher required) and a corresponding e-mail blacklist. For registration form protection, set $ras_reg_protection equal to "true". Also tweaked the audio file generation with wav files to be more robust against spambots.
2007-11-20  Version 0.2.6  Minor code tweak to the image and audio generation code so that full page generation doesn't occur unnecessarily (thanks Steve!).
2007-11-11  Version 0.2.5  Made a few minor efficiency and display tweaks, and switched up the fonts.
2007-10-13  Version 0.2.4  Tweaked a database command to be more compatible, allowed a site address override (over the WordPress site URL value), and removed some redundant code.
2007-09-30  Version 0.2.3  Fixed a bug that was blocking trackbacks and pingbacks for the past few versions (a stupid oversight on my part in the order of some code).
2007-09-26  Version 0.2.2  Fixed minor CSS detail, allowed for different folder names for the plugin, and renamed mp3 class to avoid the possibility of clashes with other class names.
2007-09-10  Version 0.2.1  Fixed a bug whereby the audio files weren't generating in all Internet Explorer setups. Fixed a bug with the "don't force registered users to enter the anti-spam word" setting. Changed table creation method so that PHP installations with low memory allocated to scripts will still be able to create the necessary tables. Also, made all display strings more easily customizable, to enhance translation support.
2007-09-03  Version 0.2.0  The plugin now uses a database for truly random words and can generate mp3 audio files of the words.
2007-07-28  Version 0.1.0  The goal of this spin-off plugin is so that people don't have to enter the words themselves (instead the plugin file creates some random text like "2fkt9s").
*/

// -----------------------------------------------------------------------------------
// Customize these parts if you want to change the defaults.

// Set this to how many letters you want the anti-spam word to be
$ras_length = 7;

// Set this to equal TRUE if you want to force registered users to enter the anti-spam text as well.
$ras_forcereg = false;

// Set this to equal TRUE if you want to allow trackbacks (but be vulnerable to trackback spam)
$ras_allowtrack = true;

// Set this to equal TRUE if you want to send all trackbacks to the moderation queue (only works if the above setting is TRUE)
$ras_modtrack = false;

// Set this to equal TRUE if you want to allow pingbacks (but be vulnerable to pingback spam)
$ras_allowping = true;

// Set this to equal TRUE if you want to send all pingbacks to the moderation queue (only works if the above setting is TRUE)
$ras_modping = false;

$ras_fontlist = array(); // <-- This line should not be changed
// List as many TrueType font(s) as you like, one per line. Drop your own font files into this plugin's directory.
// If you are using your own fonts, make sure all fonts used are about the same default size.
// If you want some fonts to be used more frequently, enter them multiple times.
// Default freeware fonts from fonts101.com
$ras_fontlist[] = "04b03.ttf";
$ras_fontlist[] = "baileysansitcbook.ttf";
$ras_fontlist[] = "decorative-stylisticblackout-regular.ttf";
$ras_fontlist[] = "okrienhmk.ttf";
$ras_fontlist[] = "pointed.ttf";
$ras_fontlist[] = "typoderm.ttf";

// Set the anti-spam image width and height.
// You may need to increase these sizes for longer words and/or bigger fonts.
$ras_imgwidth = 160;
$ras_imgheight = 50;

// Set this to TRUE if you want to use random text colors.
// If random colors are not selected, blue text will appear on a white background, and white text will appear on black background (as decided in the next option)
$ras_randomcolors = true;

// Set the background color for the anti-spam image.
// Choose either "black" or "white"
$ras_bgcolorset = "white";

// Set the border color for the anti-spam image.
// Write either major colors (red, green, blue, etc.) or enter the HTML color code (such as #C0C0C0)
$ras_borderclr = "black";

// Set this to TRUE if you prefer PNG graphics (better quality text)
// Set this to FALSE if you prefer more compatable graphics (PNG crashes IE 4; JPEG does not)
$ras_UsePngNotJpeg = false;

// Set this to TRUE if you will be editing your comments file (add this php line wherever you want the anti-spam image inserted in the comments.php file: do_action('secure_image', $post->ID); )
// Set this to FALSE if you want to use the default JavaScript positioning
$ras_manualinsert = false;

// Leave this as FALSE unless you are instructed by an error message to "The site administrator needs to manually configure his/her site address in the plugin configuration file!" In that case, enter your site address, which should be the same as the WordPress address from your Options page. Enter it in quotes like the commented line below, withOUT a trailing slash.
// $ras_siteurl = "http://yoursiteaddress/whatever";
$ras_siteurl = false;

// Set this to TRUE to enable mp3 files of the anti-spam words so that visually enabled users can comment
$ras_wav = true;

// Set this to FALSE if you know that your server doesn't have the sox filter or if you do not want to stretch the audio files by a variable rate
$ras_sox = true;

// Set this to TRUE to insert anti-spam protection to the registration form
$ras_reg_protection = false;

$ras_reg_blacklist = array();
// Registration blacklist. Enter either e-mail address or domains (like 'theblog.ca') of e-mail addresses you don't want to allow
// Be careful, because if you enter 'theblog.ca' if an e-mail address contains that string anywhere, it will be blocked
$ras_reg_blacklist[] = 'exampleaddressthatwontmatch.bc.ca';

// -----------------------------------------------------------------------------------
// Translation stuff. If your blog is in a different language, edit these fields to suit your language!
$ras_displaytext = array(); // <-- This line should not be changed

// This is the name of the label for the comment form
$ras_displaytext['label'] = "Anti-spam text: (Required)";

// These are the instructions for filling in the anti-spam word in the comment form
$ras_displaytext['instructions'] = "To prove you're a person (not a spam script), type the security text shown in the picture. <a href='#ras_security' onclick='updateSpam();' style='color:#FF0000;'>Click here to regenerate some new text</a>.";

// Error message if someone has not typed anything into the anti-spam field
$ras_displaytext['emptyfield'] = "Error: Please enter the anti-spam text.";

// Error message if the particular anti-spam image has already been used on a comment
$ras_displaytext['alreadyused'] = 'Error: The anti-spam word is invalid. Please report this error to the webmaster. Go back and refresh the page to re-submit your comment.';

// Error message if someone has typed the wrong text into the anti-spam field
$ras_displaytext['wrongfield'] = "Error: Please enter the correct anti-spam text. Press the Back button and try again.";

// Error message instructions to copy the text of the comment before pressing the back button:
$ras_displaytext['copyfield'] = "Copy your comment in case this site forces a page reload whenever you press the Back button:";

// Error message when trying to generate an audio file and the anti-spam image has already been used
$ras_displaytext['not_valid'] = 'That anti-spam number is no longer valid.';

// Text to show in an invalid image
$ras_displaytext['invalid'] = '* * * INVALID * * *';

// Error message to point the webmaster to edit the plugin configuration settings
$ras_displaytext['manually_configure'] = 'The site administrator needs to manually configure his site address in the plugin configuration file!';

// Error message if the GD Library is not installed
$ras_displaytext['no_gd'] = 'Cannot initialize new GD image stream';

if (!$ras_wav) {
// Text for the normal alt tag of the image
$ras_displaytext['alt_tag'] = 'Anti-spam image';
}
else {
// Text for the alt tag of the image for visually impaired users
$ras_displaytext['alt_tag'] = 'Click to hear an audio file of the anti-spam word';
}

// Error messages for the registration form only 

// Error if the particular anti-spam image has already been used
$ras_displaytext['register_alreadyused'] = 'ERROR: The anti-spam text is no longer valid.';

// Error message if someone has typed the wrong answer into the anti-spam field
$ras_displaytext['register_wrongfield'] = 'ERROR: Please enter the correct anti-spam text.';

// Error message if someone has typed a blocked e-mail address
$ras_displaytext['register_blocked'] = 'ERROR: That e-mail address has been blocked.';

// -----------------------------------------------------------------------------------
// You should not need to (and probably shouldn't) edit anything from here to the end.

// This makes it so that the invalid audio file pages are not indexed by search engines
$ras_displaytext['not_valid'] = '<html><head><title>' . $ras_displaytext['not_valid'] . '</title><meta name="Robots" content="noindex,nofollow"></head><body>' . $ras_displaytext['not_valid'] . '</body></html>';

// This function was modified from wp-signup-captcha
function random_text($length) {

	// this will consist of all letters and numbers
	$character_pool = "23456789bcdfghkmnpqrstwxyz";
	$character_count = strlen($character_pool) - 1;
	$code = "";
	$i = 0;
	while ($i < $length) {
		$code .= substr($character_pool, mt_rand(0, $character_count), 1);
		++$i;
		}
		return $code;
	}

// Define the path to the plugin
$ras_folder = str_replace('\\', '/', dirname(__FILE__));
$ras_thisfolder = explode('/', $ras_folder);
$ras_fontpath =  $ras_folder. '/';

// If you are calling the plugin file directly, include the file that contains important db functions
if ( ( isset($_GET['antiselect']) || isset($_GET['audioselect']) ) && file_exists('random_anti_spam.php')) {
    include_once '../../../wp-config.php';
}

global $wpdb; // Must be global for WordPress 2.5
global $ras_table; // Must be global for WordPress 2.5
// Name of the table that holds all the anti-spam words
$ras_table = $wpdb->prefix . "ras_image";

global $ras_count; // Must be global for WordPress 2.5
// Name of the table that holds the count for the anti-spam words
$ras_count = $wpdb->prefix . "ras_count";

// Generate a new anti-spam image when called by the user
if ( isset( $_GET['antinew'] ) ) {

// Generate a new word
$ras_antiselect = random_text($ras_length);

// Insert a row into the count database to generate an auto_increment number
$wpdb->query('INSERT INTO ' . $ras_count . ' (id) VALUES (NULL)');

// Get the id of the inserted count to feed to the word table and image generator
$ras_rowid = $wpdb->get_var('SELECT last_insert_id()');

// Put the random word into the database
$wpdb->query('INSERT INTO ' . $ras_table . ' (id, createtime, word) VALUES (' . $ras_rowid . ', ' . time() . ', \'' . $ras_antiselect . '\')');

// Delete the row from the count table
$wpdb->query('DELETE FROM ' . $ras_count . ' WHERE id = ' . $ras_rowid);

// Do some table admin while we can :D
if (strlen($ras_rowid) == 10) {

	// Delete all rows from the count table
	$wpdb->query('DELETE FROM ' . $ras_count);

	// Reset the table's auto increment if it's getting too huge
	$wpdb->query('ALTER TABLE ' . $ras_count . ' AUTO_INCREMENT=1');

	// Delete any anti-spam words more than a day old
	$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE ' . time() . ' > createtime + 86400');
	}
// Output the number to send back to the user display
die("$ras_rowid");
}

// Output the antispam audio file
if( isset( $_GET['audioselect'] ) )
{
	// Include the wav merging class
	include $ras_fontpath . 'wav_join.php';

	// Validate the input values
	$ras_audioselect = intval( $_GET['audioselect'] );

	// Get the word from the database
	$ras_antispam = $wpdb->get_var('SELECT word FROM ' . $ras_table . ' WHERE id = ' . $ras_audioselect);
	if (is_null($ras_antispam)) die($ras_displaytext['not_valid']);

	// Get the word
	$petersword = strtolower($ras_antispam);

	// Count the number of letters in the word
	$word_count = strlen($petersword);

	// Write the function str_split if it doesn't exist (in PHP 4)
	if (!function_exists('str_split')) {
		function str_split($string) {
			global $word_count;
			$letters = array();
			for ($i = 0; $i < $word_count; ++$i) {
				$letters[$i] = substr($string, $i, 1);
			}
			return $letters;
		}
	}

	// Split the word into an array of individual letters
	$ras_wavs = str_split($petersword);

	// Merge the wav files for this word
	$ras_one_wav = joinwavs($ras_wavs);

	// Create a temporary file for this word
	$ras_audioinput = tempnam($ras_fontpath, 'ras_');
	$ras_audiooutput = $ras_audioinput . '1.wav';
	$ras_handle=fopen($ras_audioinput, "w");
	fwrite($ras_handle, $ras_one_wav);
	fclose($ras_handle);

	// Clear the variable out since the sound is now in a temporary file
	unset($ras_one_wav);

    if ($ras_sox) {
        // Make use of the sox filter, if available, to make it hard for bots to crack the audio file

        // This will determine a stretch factor with 20 different possibilities
        $stretch_random = sprintf("%02d", rand( 0, 20));

        // Stretch the audio file
        @exec('sox ' . $ras_audioinput . ' ' . $ras_audiooutput . ' stretch 1.' . $stretch_random . ' > /dev/null 2>&1');
    }

	// If the sox filter was available, use the stretched file. Otherwise, use the original file
	if (file_exists($ras_audiooutput)) {
		$ras_one_wav = file_get_contents($ras_audiooutput);
	}
	else {
		$ras_one_wav = file_get_contents($ras_audioinput);
	}

	// Spit it the file out to the browser	
	output('output.wav', $ras_one_wav);

	// Delete the files
	if (file_exists($ras_audioinput)) unlink($ras_audioinput);
	if (file_exists($ras_audiooutput)) unlink($ras_audiooutput);

	// Prevent any more display to the page
	die();
}


// Output the antispam image
elseif( isset( $_GET['antiselect'] ) )
{
	// Pick a random font to use
	$ras_font = $ras_fontpath . $ras_fontlist[ rand( 0, count( $ras_fontlist ) - 1 ) ];

	// Set the default colors for when random text colors are not selected
	if ( $ras_bgcolorset == "white") {
	$ras_textcolor = array( 0, 0, 255 ); // blue text
	$ras_bgcolor = array( 255, 255, 255); // white background
	}
	else {
	$ras_textcolor = array( 255, 255, 255 ); // white text
	$ras_bgcolor = array( 0, 0, 0); // black background
	}

	// If selected, pick a random color for the antispam word text
	if( $ras_randomcolors )
	{
		$ras_rand = rand( 0, 4 );
		switch( $ras_bgcolorset )
			{
				case "white":
					$ras_textcolorchoice[0] = array ( 0, 0, 255 ); // blue
					$ras_textcolorchoice[1] = array ( 0, 153, 0 ); // greenish
					$ras_textcolorchoice[2] = array ( 204, 0, 0 ); // reddish
					$ras_textcolorchoice[3] = array ( 203, 0, 154 ); // purplish
					$ras_textcolorchoice[4] = array ( 0, 0, 0 ); // black
					$ras_textcolor = $ras_textcolorchoice[$ras_rand];
					break;
				default:
					$ras_textcolorchoice[0] = array ( 255, 255, 0 ); // yellow
					$ras_textcolorchoice[1] = array ( 0, 255, 255 ); // blueish
					$ras_textcolorchoice[2] = array ( 255, 153, 204 ); // pinkish
					$ras_textcolorchoice[3] = array ( 102, 255, 102 ); // greenish
					$ras_textcolorchoice[4] = array ( 255, 255, 255 ); // white
					$ras_textcolor = $ras_textcolorchoice[$ras_rand];
					break;
			}
	}

	// Validate the input values
	$ras_antiselect = intval( $_GET['antiselect'] );

	// Get the word from the database
	$ras_antispam = $wpdb->get_var('SELECT word FROM ' . $ras_table . ' WHERE id = ' . $ras_antiselect);
	if (is_null($ras_antispam)) $ras_antispam = $ras_displaytext['invalid'];

	// Start building the image
	$ras_image = @imagecreate( $ras_imgwidth, $ras_imgheight ) or die($ras_displaytext['no_gd']);
	$ras_bgcolor = imagecolorallocate( $ras_image, $ras_bgcolor[0], $ras_bgcolor[1], $ras_bgcolor[2] );
	$ras_fontcolor = imagecolorallocate( $ras_image, $ras_textcolor[0], $ras_textcolor[1], $ras_textcolor[2] );

	// Check for freetype lib, if not found default to ugly built in capability using imagechar (Lee's mod)
	// Also check that the chosen TrueType font is available
	if( function_exists( 'imagettftext' ) && file_exists( $ras_font ) )
	{
		$ras_angle = 4; // Degrees to tilt the text
		$ras_offset = 10; // Pixels to offset the text from the border
		$ras_fontsize = 28; // Default font size for the anti-spam image
		$ras_imagebox = imagettfbbox($ras_fontsize, $ras_angle, $ras_font, $ras_antispam);
		$ras_boxwidth = $ras_imagebox[2] - $ras_imagebox[0];
		$ras_boxheight = $ras_imagebox[1] - $ras_imagebox[7];
		
		// if the text width is too big for the image, decrease the font size to a certain extent (best practice is of course not to use really long words!
		while ($ras_boxwidth > $ras_imgwidth - $ras_offset && $ras_fontsize > 12) {
			$ras_fontsize = $ras_fontsize - 3;
			$ras_imagebox = imagettfbbox($ras_fontsize, $ras_angle, $ras_font, $ras_antispam);
			$ras_boxwidth = $ras_imagebox[2] - $ras_imagebox[0];
		}

		// if the text height too big for the image, decrease the font size to a certain extent (best practice is of course not to use really long words!
		while ($ras_boxheight > $ras_imgheight - $ras_offset && $ras_fontsize > 12) {
			$ras_fontsize = $ras_fontsize - 3;
			$ras_imagebox = imagettfbbox($ras_fontsize, $ras_angle, $ras_font, $ras_antispam);
			$ras_boxheight = $ras_imagebox[1] - $ras_imagebox[7];
		}

		// Use png is available, since it produces clearer text images
		if( function_exists( 'imagepng' ) && $ras_UsePngNotJpeg )
		{
			imagettftext( $ras_image, $ras_fontsize, $ras_angle, $ras_offset, $ras_imgheight - $ras_offset, $ras_fontcolor, $ras_font, $ras_antispam );
			header( "Content-type: image/png" );
			imagepng( $ras_image );
		} else {
			imagettftext( $ras_image, $ras_fontsize, $ras_angle, $ras_offset, $ras_imgheight - $ras_offset, $ras_fontcolor, $ras_font, $ras_antispam );
			header( "Content-type: image/jpeg" );
			imagejpeg( $ras_image );
		}
	} else {
		$ras_fontsize = 5; // 1, 2, 3, 4 or 5 (higher numbers correspond to larger font sizes)
		$tmp_len = strlen( $ras_antispam );
		for( $tmp_count = 0; $tmp_count < $tmp_len; $tmp_count++ )
		{
		   $tmp_xpos = $tmp_count * imagefontwidth( $ras_fontsize ) + 20;
		   $tmp_ypos = 10;
		   imagechar( $ras_image, $ras_fontsize, $tmp_xpos, $tmp_ypos, $ras_antispam, $ras_fontcolor );
		   $ras_antispam = substr( $ras_antispam, 1);   
		}
		header("Content-Type: image/gif");
		imagegif( $ras_image );
	} // end if
	imagedestroy( $ras_image );
	die();
} else {

	// Determine the blog url to this script
	$ras_message = "<small>{$ras_displaytext['instructions']}</small>";
	if ( function_exists(get_option) && !$ras_siteurl) {
		$ras_siteurl = get_option('siteurl');
		}
	if ( !$ras_siteurl ) {
		$ras_message = "<h1>{$ras_displaytext['manually_configure']}</h1>";
		}
	$ras_myurl = $ras_siteurl;

	class RandomAntiSpam
	{
		function RandomAntiSpam()
		{
			global $ras_manualinsert;
			add_action( 'secure_image', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			if (! $ras_manualinsert ) {
				add_action( 'comment_form', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			}
			add_filter( 'preprocess_comment', array( &$this, 'comment_post') );    // add post comment post security code check
		}

		function comment_form()
		{
			global $ras_forcereg, $user_ID, $ras_myurl, $ras_thisfolder, $ras_imgheight, $ras_imgwidth, $ras_limitcolor, $ras_manualinsert, $ras_borderclr, $ras_message, $ras_displaytext, $ras_table, $ras_count, $ras_wav, $wpdb, $ras_length;
			// If the user is logged in, don't prompt for code
			if( ! $ras_forcereg && intval( $user_ID ) > 0 )
			{
				return( $post_ID );
			}

			$ras_antiselect = random_text($ras_length);

			// Insert a row into the count database to generate an auto_increment number
			$wpdb->query('INSERT INTO ' . $ras_count . ' (id) VALUES (NULL)');

			// Get the id of the inserted count to feed to the word table and image generator
			$ras_rowid = $wpdb->get_var('SELECT last_insert_id()');

			// Put the random word into the database
			$wpdb->query('INSERT INTO ' . $ras_table . ' (id, createtime, word) VALUES (' . $ras_rowid . ', ' . time() . ', \'' . $ras_antiselect . '\')');

			// Delete the row from the count table
			$wpdb->query('DELETE FROM ' . $ras_count . ' WHERE id = ' . $ras_rowid);

			// Do some table admin while we can :D
			if (strlen($ras_rowid) == 10) {

				// Delete all rows from the count table
				$wpdb->query('DELETE FROM ' . $ras_count);

				// Reset the table's auto increment if it's getting too huge
				$wpdb->query('ALTER TABLE ' . $ras_count . ' AUTO_INCREMENT=1');

				// Delete any anti-spam words more than a day old
				$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE ' . time() . ' > createtime + 86400');
			}
			// The JavaScript that updates the anti-spam image

			print '<script language="javascript" type="text/javascript">' . "\n";
			print '// Code modified from Peter\'s ajax date script at http://www.theblog.ca/?p=148' . "\n";
			print '// The date script modified code from a tutorial here: http://www.webpasties.com/xmlHttpRequest/xmlHttpRequest_tutorial_1.html' . "\n\n";

			print "// Update the code that displays the anti-spam image
			var url = \"?antinew\"; // The server-side script
			function handleHttpResponse() {
			  if (http.readyState == 4) {
			  if (http.responseText.indexOf('invalid') == -1) {
			    results = http.responseText;
			    document.getElementById('ras_match').value = results;
			    document.getElementById('ras_image').src = '" . $ras_myurl . '/wp-content/plugins/' . end($ras_thisfolder) . "/random_anti_spam.php?antiselect=' + results;
			    document.getElementById('ras_link').href = '" . $ras_myurl . '/wp-content/plugins/' . end($ras_thisfolder) . "/random_anti_spam.php?audioselect=' + results;
			    isWorking = false;
			    }
			  }
			}

			var isWorking = false;\n\n";

			print "// Call up the script that generates the new id\n
			function updateSpam() {
			if (!isWorking && http) {
			  http.open(\"GET\", url, true);
			  http.onreadystatechange = handleHttpResponse;
			isWorking = true;
			  http.send(null);
			}
			}\n\n";

			print "function getHTTPObject() {
			var xmlhttp;
			/*@cc_on
			@if (@_jscript_version >= 5)
			try { xmlhttp = new ActiveXObject(\"Msxml2.XMLHTTP\"); }
			catch (e) {
			try { xmlhttp = new ActiveXObject(\"Microsoft.XMLHTTP\"); }
			catch (E) { xmlhttp = false; } }
			@else xmlhttp = false;
			@end @*/
			if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
			try { xmlhttp = new XMLHttpRequest(); }
			catch (e) { xmlhttp = false; } }
			return xmlhttp;
			}
			var http = getHTTPObject(); // We create the HTTP Object 
			</script>\n";


			echo( "\t\t\t".'<div style="display:block;" id="secureimgdiv">'."\n\t\t\t\t" );
			echo( "<p><a name=\"ras_security\"></a><label for=\"securitycode\">{$ras_displaytext['label']}</label><span style=\"color:#FF0000;\">*</span><br />\n\t\t\t\t" );
			echo( $ras_message . "<br />\n\t\t\t\t" );
			echo( '<input type="text" name="securitycode" id="securitycode" size="30" tabindex="4" />'."\n\t\t\t\t" );
			echo( '<input type="hidden" name="matchthis" id="ras_match" value="' . $ras_rowid . "\" />\n\t\t\t\t" );
			if ($ras_wav) echo( '<a id="ras_link" href="' . $ras_myurl . '/wp-content/plugins/' . end($ras_thisfolder) . '/random_anti_spam.php?audioselect=' . $ras_rowid . '" rel="nofollow" title="' . $ras_displaytext['alt_tag'] . '">' . "\n\t\t\t\t" );
			echo( '<img id="ras_image" src="' . $ras_myurl . '/wp-content/plugins/' . end($ras_thisfolder) . '/random_anti_spam.php?antiselect=' . $ras_rowid . '" ' );
			echo( 'alt="' . $ras_displaytext['alt_tag'] . '" ' );
			echo( 'style="border:1px solid ' . $ras_borderclr . ';vertical-align:top;' );
			echo( 'height:' . $ras_imgheight .'px;width:' . $ras_imgwidth . 'px;" />' );
			if ($ras_wav) echo( '</a>');
			echo( "</p>\n\t\t\t" );
			echo( "</div>\n\t\t\t" );
if ( ! $ras_manualinsert ) {
			echo"<script language='JavaScript' type='text/javascript'>
			<!--
				var urlinput = document.getElementById(\"url\");
				var submitp = urlinput.parentNode;
				var substitution2 = document.getElementById(\"secureimgdiv\");
				submitp.appendChild(substitution2, urlinput);
			// -->
			</script>\n";
}
			return( $post_ID );
		}

		function comment_post( $incoming_comment )
		{
			global $_POST, $ras_text, $ras_textcount, $ras_forcereg, $user_ID, $ras_allowtrack, $ras_allowping, $ras_modping, $ras_modtrack, $ras_displaytext, $ras_length, $ras_table, $wpdb;
			// Validate the form input values
			if( isset( $_POST['securitycode'] ) )
			{
				$securitycode = substr( strval( $_POST['securitycode'] ), 0, 50 );
			} else {
				$securitycode = '';
			}
			if( isset( $_POST['matchthis'] ) )
			{
				$matchnum = intval( $_POST['matchthis'] );
			} else {
				$matchnum = 0;
			}

			// If the user is not logged in check the security code
			if( $ras_forcereg || ! isset( $user_ID ) )
			{
				$istrackping = $incoming_comment['comment_type'];
				$commentbody = $incoming_comment['comment_content'];
				if ( $istrackping == 'pingback' && $ras_allowping ) {

					// Send all pingbacks to a moderation queue?
					if ($ras_modping)
						add_filter('pre_comment_approved', create_function('$mod_ping', 'return \'0\';'));
				}
				elseif ( $istrackping == 'trackback' && $ras_allowtrack ) {

					// Send all trackbacks to a moderation queue?
					if ($ras_modtrack)
						add_filter('pre_comment_approved', create_function('$mod_track', 'return \'0\';'));
				}
				else
				{
					if ( $securitycode == '' )
					{
						die("<p>{$ras_displaytext['emptyfield']}</p>\n<p>{$ras_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");
					}

					// Get the anti-spam word from the database
					$matchthis = $wpdb->get_var('SELECT word FROM ' . $ras_table . ' WHERE id = ' . $matchnum);

					// If this row doesn't exist, say something
					if (is_null($matchthis)) die("<p>{$ras_displaytext['alreadyused']}</p>\n<p>{$ras_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");

					if ( strtolower( $matchthis ) != strtolower( $securitycode ) )
					{
						die("<p>{$ras_displaytext['wrongfield']}</p>\n<p>{$ras_displaytext['copyfield']}</p>\n<textarea cols=\"100%\" rows=\"10\" onclick=\"this.select();\" readonly=\"true\">$commentbody</textarea>");
					} else {
						// The word matched, so delete the row for the anti-spam word so that it cannot be used again
						$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE id = ' . $matchnum);
						unset( $matchthis );

						// Do some more table admin while we can :D
						// Delete any anti-spam words more than a day old
						$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE ' . time() . ' > createtime + 86400');
					}
				}
			}
			return( $incoming_comment );
		}
	}
	new RandomAntiSpam();

// Add some troubleshooting to the Manage tab

function ras_manage() {
global $ras_siteurl;
print "<div class=\"wrap\">\n";
print "<h2>Peter's Random Anti-Spam</h2>\n";
print "<p>This page will diagnose your server setup to see how it relates to the performance of this plugin. Remember that all actual settings for this plugin can be changed under the Plugins &gt; Plugin Editor menu.</p>\n";
print "<hr />\n";

print "<p><strong>GD library</strong><br />\n";
if (function_exists(imagejpeg)) {
print "Yay! The GD library is installed. This is the most important thing to be able to use this plugin. You might still need to do some tweaking to the settings of course.</p>\n";
}
else {
print "<font color=\"red\">The GD library is not installed. This plugin will not work without it. Ask your webhost to install this library (or install it if you manage your own server).</font></p>\n";
}

print "<p><strong>FreeType</strong><br />\n";
if (function_exists(imagettftext)) {
print "Yay! The FreeType library is installed. The anti-spam image should display using the uploaded fonts.</p>\n";
}
else {
print "<font color=\"red\">The FreeType library is not installed. The anti-spam image should still display, but with only a plain font. Ask your webhost to install this library (or install it if you manage your own server) if you want to be able to use the uploaded fonts.</font></p>\n";
}

print "<p><strong>get_option</strong><br />\n";
if (function_exists(get_option)) {
print "Yay! The script should be able to automatically figure out your blog address.</p>\n";
}
elseif ($ras_siteurl) {
print "The script cannot automatically figure out your blog address, but you have entered it manually. This should be the correct address to your blog: $ras_siteurl. Otherwise, please make sure you correct the \$ras_siteurl value in the plugin file (something like <em>http://www.yoursiteaddress.com/whatever</em>).";
}
else {
print "<font color=\"red\">For some reason, the script cannot access your blog address. Make sure you edit the \$ras_siteurl value in the plugin file (something like <em>http://www.yoursiteaddress.com/whatever</em>).</font></p>\n";
}

print "<hr />\n";
print "<p>Need more help? Post a comment on the <a href=\"http://www.theblog.ca/?page_id=138\" title=\"Where all information is posted about the plugin\">Peter's Random Anti-Spam page</a>.</p>\n";
print "</div>\n";
}

function ras_adminmenu() {
	add_management_page('Random anti-spam', 'Random anti-spam', 7, 'random_anti_spam.php', 'ras_manage');
}

add_action('admin_menu','ras_adminmenu',1);

// Install the anti-spam word database table upon plugin activation

function ras_install () {
	global $wpdb, $ras_table, $ras_count;

	// Add the table to hold the anti-spam words
	if($wpdb->get_var("SHOW TABLES LIKE '$ras_table'") != $ras_table) {
		$sql = "CREATE TABLE " . $ras_table . " (
	  	id int(10) NOT NULL,
	  	createtime int(10) NOT NULL,
	  	word VARCHAR(20) NOT NULL,
	  	UNIQUE KEY id (id)
		);";
      	$wpdb->query($sql);
	}

	// Add the table to hold the count for the anti-spam words
	if($wpdb->get_var("SHOW TABLES LIKE '$ras_count'") != $ras_count) {
		$sql = "CREATE TABLE " . $ras_count . " (
	  	id int(10) NOT NULL AUTO_INCREMENT,
	  	UNIQUE KEY id (id)
		);";
      	$wpdb->query($sql);
	}
      	
}

// Remove the anti-spam word database table upon plugin de-activation

function ras_uninstall () {
	global $wpdb, $ras_table, $ras_count;

	if($wpdb->get_var("SHOW TABLES LIKE '$ras_table'") == $ras_table) {
		$sql = "DROP TABLE " . $ras_table;
		$wpdb->query($sql);
	}

	if($wpdb->get_var("SHOW TABLES LIKE '$ras_count'") == $ras_count) {
		$sql = "DROP TABLE " . $ras_count;
		$wpdb->query($sql);
	}
}

add_action('activate_' . end($ras_thisfolder) . '/random_anti_spam.php', 'ras_install');
add_action('deactivate_' . end($ras_thisfolder) . '/random_anti_spam.php', 'ras_uninstall');

function ras_register_form() {
	global $ras_length, $ras_myurl, $ras_imgheight, $ras_imgwidth, $ras_limitcolor, $ras_borderclr, $ras_message, $ras_displaytext, $ras_table, $ras_count, $ras_wav, $wpdb;

	$ras_antiselect = random_text($ras_length);

	// Insert a row into the count database to generate an auto_increment number
	$wpdb->query('INSERT INTO ' . $ras_count . ' (id) VALUES (NULL)');

	// Get the id of the inserted count to feed to the word table and image generator
	$ras_rowid = $wpdb->get_var('SELECT last_insert_id()');

	// Put the random word into the database
	$wpdb->query('INSERT INTO ' . $ras_table . ' (id, createtime, word) VALUES (' . $ras_rowid . ', ' . time() . ', \'' . $ras_antiselect . '\')');
	
	// Delete the row from the count table
	$wpdb->query('DELETE FROM ' . $ras_count . ' WHERE id = ' . $ras_rowid);

	// Do some table admin while we can :D
	if (strlen($ras_rowid) == 10) {

		// Delete all rows from the count table
		$wpdb->query('DELETE FROM ' . $ras_count);

		// Reset the table's auto increment if it's getting too huge
		$wpdb->query('ALTER TABLE ' . $ras_count . ' AUTO_INCREMENT=1');

		// Delete any anti-spam words more than a day old
		$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE ' . time() . ' > createtime + 86400');
	}

	// The JavaScript that updates the anti-spam image

	print '<script language="javascript" type="text/javascript">' . "\n";
	print '// Code modified from Peter\'s ajax date script at http://www.theblog.ca/?p=148' . "\n";
	print '// The date script modified code from a tutorial here: http://www.webpasties.com/xmlHttpRequest/xmlHttpRequest_tutorial_1.html' . "\n\n";

	print "// Update the code that displays the anti-spam image
	var url = \"?antinew\"; // The server-side script
	function handleHttpResponse() {
	  if (http.readyState == 4) {
	  if (http.responseText.indexOf('invalid') == -1) {
	    results = http.responseText;
	    document.getElementById('ras_match').value = results;
	    document.getElementById('ras_image').src = '" . $ras_myurl . "?antiselect=' + results;
	    document.getElementById('ras_link').href = '" . $ras_myurl . "?audioselect=' + results;
	    isWorking = false;
	    }
	  }
	}
		var isWorking = false;\n\n";
		print "// Call up the script that generates the new id\n
	function updateSpam() {
	if (!isWorking && http) {
	  http.open(\"GET\", url, true);
	  http.onreadystatechange = handleHttpResponse;
	isWorking = true;
	  http.send(null);
	}
	}\n\n";
		print "function getHTTPObject() {
	var xmlhttp;
	/*@cc_on
	@if (@_jscript_version >= 5)
	try { xmlhttp = new ActiveXObject(\"Msxml2.XMLHTTP\"); }
	catch (e) {
	try { xmlhttp = new ActiveXObject(\"Microsoft.XMLHTTP\"); }
	catch (E) { xmlhttp = false; } }
	@else xmlhttp = false;
	@end @*/
	if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
	try { xmlhttp = new XMLHttpRequest(); }
	catch (e) { xmlhttp = false; } }
	return xmlhttp;
	}
	var http = getHTTPObject(); // We create the HTTP Object 
	</script>\n";

	echo( "\t\t\t".'<div style="display:block;" id="secureimgdiv">'."\n\t\t\t\t" );
	echo( "<p><a name=\"ras_security\"></a><label for=\"securitycode\">{$ras_displaytext['label']}</label><span style=\"color:#FF0000;\">*</span><br />\n\t\t\t\t" );
	echo( $ras_message . "<br />\n\t\t\t\t" );
	echo( '<input type="text" name="securitycode" id="securitycode" size="30" tabindex="4" />'."\n\t\t\t\t" );
	echo( '<input type="hidden" name="matchthis" id="ras_match" value="' . $ras_rowid . "\" />\n\t\t\t\t" );
	if ($ras_wav) echo( '<a id="ras_link" href="' . $ras_myurl . '?audioselect=' . $ras_rowid . '" title="' . $ras_displaytext['alt_tag'] . '">' );
	echo( '<img id="ras_image" src="' . $ras_myurl . '?antiselect=' . $ras_rowid . '" ' );
	echo( 'alt="' . $ras_displaytext['alt_tag'] . '" ' );
	echo( 'style="border:1px solid ' . $ras_borderclr . ';vertical-align:top;' );
	echo( 'height:' . $ras_imgheight .'px;width:' . $ras_imgwidth . 'px;" />' );
	if ($ras_wav) echo( '</a>');
	echo( "</p>\n\t\t\t" );
	echo( "</div>\n\t\t\t" );
}

function ras_register_post() {
	global $errors, $_POST, $ras_text, $ras_textcount, $user_ID, $ras_displaytext, $ras_table, $wpdb, $ras_reg_blacklist;

		// Validate the form input values
		if( isset( $_POST['securitycode'] ) )
		{
			$securitycode = substr( strval( $_POST['securitycode'] ), 0, 50 );
		} else {
			$securitycode = '';
		}
		if( isset( $_POST['matchthis'] ) )
		{
			$matchnum = intval( $_POST['matchthis'] );
		} else {
			$matchnum = 0;
		}

		if ( $securitycode == '' )
		{			
			$errors['ras_register'] = $ras_displaytext['emptyfield'];
		}
		else {
			// Get the anti-spam word from the database
			$matchthis = $wpdb->get_var('SELECT word FROM ' . $ras_table . ' WHERE id = ' . $matchnum);

			// If this row doesn't exist, say something
			if (is_null($matchthis)) {
				$errors['ras_register'] = $ras_displaytext['register_alreadyused'];
			}
			elseif ( strtolower( $matchthis ) != strtolower( $securitycode ) ) {
				$errors['ras_register'] = $ras_displaytext['register_wrongfield'];
			}
			else {
				// The word matched, so delete the row for the anti-spam word so that it cannot be used again
				$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE id = ' . $matchnum);
				unset( $matchthis );

				// Do some more table admin while we can :D
				// Delete any anti-spam words more than a day old
				$wpdb->query('DELETE FROM ' . $ras_table . ' WHERE ' . time() . ' > createtime + 86400');
			}
		}
		foreach ($ras_reg_blacklist as $ras_blacklist) {
			if (stristr($_POST['user_email'], $ras_blacklist)) {
				$errors['ras_register'] = $ras_displaytext['register_blocked'];
				break;
			}
		}
	}

// Add registration protection to the appropriate hooks only if it has been enabled in this plugin's settings
if ($ras_reg_protection) {
	add_action('register_form', 'ras_register_form');
	add_action('register_post', 'ras_register_post');
	}
}
?>